@@ -6,13 +6,13 @@
  */
 
 #include "configuration.h"
+#include <algorithm>
 #include <cassert>
-#include <exception>
 #include <list>
 #include <memory>
 #include <stdexcept>
 #include <unordered_map>
-#include "fcitx-utils/standardpath.h"
+#include "fcitx-utils/stringutils.h"
 
 namespace fcitx {
 class ConfigurationPrivate {
@@ -24,7 +24,7 @@ public:
 Configuration::Configuration()
     : d_ptr(std::make_unique<ConfigurationPrivate>()) {}
 
-Configuration::~Configuration() {}
+Configuration::~Configuration() = default;
 
 void Configuration::dumpDescription(RawConfig &config) const {
     return dumpDescriptionImpl(config, {});
@@ -55,7 +55,7 @@ void Configuration::dumpDescriptionImpl(
             auto subConfigPath = parentPaths;
             subConfigPath.push_back(option->path());
             std::string subTypeName = subConfig->typeName();
-            auto oldTypeName = descConfigPtr->valueByPath("Type");
+            const auto *oldTypeName = descConfigPtr->valueByPath("Type");
             // Replace the "Type" with the full name we want.
             // Path$To$Value$TypeName
             if (oldTypeName &&
@@ -65,7 +65,7 @@ void Configuration::dumpDescriptionImpl(
                 newTypeName.append(stringutils::join(subConfigPath, '$'));
                 newTypeName.append("$");
                 newTypeName.append(subTypeName);
-                descConfigPtr->setValueByPath("Type", newTypeName);
+                descConfigPtr->setValueByPath("Type", std::move(newTypeName));
             }
             subConfigs.emplace_back(subConfigPath, std::move(subConfig));
         }
@@ -79,14 +79,15 @@ void Configuration::dumpDescriptionImpl(
 
 bool Configuration::compareHelper(const Configuration &other) const {
     FCITX_D();
-    for (const auto &path : d->optionsOrder_) {
-        auto optionIter = d->options_.find(path);
-        assert(optionIter != d->options_.end());
-        auto otherOptionIter = other.d_func()->options_.find(path);
-        if (*optionIter->second != *otherOptionIter->second) {
-            return false;
-        }
-    }
+    return std::all_of(
+        d->optionsOrder_.begin(), d->optionsOrder_.end(),
+        [d, &other](const auto &path) {
+            auto optionIter = d->options_.find(path);
+            assert(optionIter != d->options_.end());
+            auto otherOptionIter = other.d_func()->options_.find(path);
+            assert(otherOptionIter != other.d_func()->options_.end());
+            return *optionIter->second == *otherOptionIter->second;
+        });
     return true;
 }
 
@@ -150,9 +151,10 @@ void Configuration::syncDefaultValueToCurrent() {
         // Unfortunately on certain system OptionBaseV2 doesn't have key
         // function emit type info, so we have to add OptionBaseV3 with a
         // non-abstract virtual funciton.
-        if (auto optionV3 = dynamic_cast<OptionBaseV3 *>(iter->second)) {
+        if (auto *optionV3 = dynamic_cast<OptionBaseV3 *>(iter->second)) {
             optionV3->syncDefaultValueToCurrent();
-        } else if (auto optionV2 = dynamic_cast<OptionBaseV2 *>(iter->second)) {
+        } else if (auto *optionV2 =
+                       dynamic_cast<OptionBaseV2 *>(iter->second)) {
             optionV2->syncDefaultValueToCurrent();
         }
     }
