@@ -57,6 +57,29 @@ public:
     FCITX_NODISCARD std::unique_ptr<EventSource>
     addPostEvent(EventCallback callback);
 
+    /**
+     * Add an async event that is safe to be triggered from another thread.
+     *
+     * To ensure safe usage of this event:
+     * 1. Do not change event to disable, if there may be pending call to send()
+     * 2. Due to (1), if it is oneshot, ensure you only call send() once.
+     * 3. Join all the possible pending thread that may call send(), before
+     *    destructing the event.
+     * 4. Like other event, the event should be only created/destructed on the
+     *    event loop thread.
+     *
+     * EventDispatcher uses this event internally and provides an easier and
+     * safer interface to use.
+     *
+     * @see EventDispatcher
+     *
+     * @param callback callback function
+     * @return async event source
+     * @since 5.1.13
+     */
+    FCITX_NODISCARD std::unique_ptr<EventSourceAsync>
+    addAsyncEvent(EventCallback callback);
+
     /**
      * Set an external event loop implementation.
      *
