@@ -5,16 +5,19 @@
  *
  */
 #include "misc.h"
-#include <sys/wait.h>
 #include <unistd.h>
 #include <cstdio>
 #include <string>
 #include <vector>
-#include <fmt/format.h>
+#include <format>
 #include "fs.h"
 #include "log.h"
 #include "misc_p.h"
 
+#ifdef HAVE_SYS_WAIT_H
+#include <sys/wait.h>
+#endif
+
 #if defined(LIBKVM_FOUND)
 #include <fcntl.h>
 #include <kvm.h>
@@ -34,7 +37,7 @@ namespace fcitx {
 
 void startProcess(const std::vector<std::string> &args,
                   const std::string &workingDirectory) {
-#if defined(__APPLE__) && TARGET_OS_IPHONE
+#if defined(_WIN32) || (defined(__APPLE__) && TARGET_OS_IPHONE)
     FCITX_UNUSED(args);
     FCITX_UNUSED(workingDirectory);
     FCITX_ERROR() << "Not implemented";
@@ -137,8 +140,11 @@ std::string getProcessName(pid_t pid) {
     }
     return result;
 #endif
+#elif defined(_WIN32)
+    FCITX_UNUSED(pid);
+    return {};
 #else
-    auto path = fmt::format("/proc/{}/exe", pid);
+    auto path = std::format("/proc/{}/exe", pid);
     if (auto link = fs::readlink(path)) {
         return fs::baseName(*link);
     }
@@ -146,6 +152,8 @@ std::string getProcessName(pid_t pid) {
 #endif
 }
 
+#ifndef _WIN32
+
 ssize_t getline(UniqueCPtr<char> &lineptr, size_t *n, std::FILE *stream) {
     auto *lineRawPtr = lineptr.release();
     auto ret = getline(&lineRawPtr, n, stream);
@@ -153,6 +161,8 @@ ssize_t getline(UniqueCPtr<char> &lineptr, size_t *n, std::FILE *stream) {
     return ret;
 }
 
+#endif
+
 bool isInFlatpak() {
 #ifdef __APPLE__
     return false;
