@@ -1,18 +1,21 @@
-#ifndef WL_SUBCOMPOSITOR
-#define WL_SUBCOMPOSITOR
-#include <memory>
+#ifndef WL_SUBCOMPOSITOR_H_
+#define WL_SUBCOMPOSITOR_H_
+#include <cstdint>
 #include <wayland-client.h>
-#include "fcitx-utils/signals.h"
+#include <wayland-util.h>
+#include "fcitx-utils/misc.h"
 namespace fcitx::wayland {
+
 class WlSubsurface;
 class WlSurface;
+
 class WlSubcompositor final {
 public:
     static constexpr const char *interface = "wl_subcompositor";
     static constexpr const wl_interface *const wlInterface =
         &wl_subcompositor_interface;
     static constexpr const uint32_t version = 1;
-    typedef wl_subcompositor wlType;
+    using wlType = wl_subcompositor;
     operator wl_subcompositor *() { return data_.get(); }
     WlSubcompositor(wlType *data);
     WlSubcompositor(WlSubcompositor &&other) noexcept = delete;
@@ -24,6 +27,7 @@ public:
 
 private:
     static void destructor(wl_subcompositor *);
+
     uint32_t version_;
     void *userData_ = nullptr;
     UniqueCPtr<wl_subcompositor, &destructor> data_;
@@ -31,5 +35,7 @@ private:
 static inline wl_subcompositor *rawPointer(WlSubcompositor *p) {
     return p ? static_cast<wl_subcompositor *>(*p) : nullptr;
 }
+
 } // namespace fcitx::wayland
-#endif
+
+#endif // WL_SUBCOMPOSITOR_H_
