@@ -1,19 +1,24 @@
-#ifndef ZWP_TABLET_PAD_V2
-#define ZWP_TABLET_PAD_V2
+#ifndef ZWP_TABLET_PAD_V2_H_
+#define ZWP_TABLET_PAD_V2_H_
+#include <cstdint>
 #include <wayland-client.h>
+#include <wayland-util.h>
+#include "fcitx-utils/misc.h"
 #include "fcitx-utils/signals.h"
-#include "wayland-tablet-client-protocol.h"
+#include "wayland-tablet-unstable-v2-client-protocol.h" // IWYU pragma: export
 namespace fcitx::wayland {
+
 class WlSurface;
 class ZwpTabletPadGroupV2;
 class ZwpTabletV2;
+
 class ZwpTabletPadV2 final {
 public:
     static constexpr const char *interface = "zwp_tablet_pad_v2";
     static constexpr const wl_interface *const wlInterface =
         &zwp_tablet_pad_v2_interface;
     static constexpr const uint32_t version = 1;
-    typedef zwp_tablet_pad_v2 wlType;
+    using wlType = zwp_tablet_pad_v2;
     operator zwp_tablet_pad_v2 *() { return data_.get(); }
     ZwpTabletPadV2(wlType *data);
     ZwpTabletPadV2(ZwpTabletPadV2 &&other) noexcept = delete;
@@ -22,6 +27,7 @@ public:
     void *userData() const { return userData_; }
     void setUserData(void *userData) { userData_ = userData; }
     void setFeedback(uint32_t button, const char *description, uint32_t serial);
+
     auto &group() { return groupSignal_; }
     auto &path() { return pathSignal_; }
     auto &buttons() { return buttonsSignal_; }
@@ -42,6 +48,7 @@ private:
     fcitx::Signal<void(uint32_t, ZwpTabletV2 *, WlSurface *)> enterSignal_;
     fcitx::Signal<void(uint32_t, WlSurface *)> leaveSignal_;
     fcitx::Signal<void()> removedSignal_;
+
     uint32_t version_;
     void *userData_ = nullptr;
     UniqueCPtr<zwp_tablet_pad_v2, &destructor> data_;
@@ -49,5 +56,7 @@ private:
 static inline zwp_tablet_pad_v2 *rawPointer(ZwpTabletPadV2 *p) {
     return p ? static_cast<zwp_tablet_pad_v2 *>(*p) : nullptr;
 }
+
 } // namespace fcitx::wayland
-#endif
+
+#endif // ZWP_TABLET_PAD_V2_H_
