@@ -9,8 +9,8 @@
 
 #include <fcitx/addoninstance.h>
 #include <fcitx/event.h>
+#include <fcitx/fcitxcore_export.h>
 #include <fcitx/inputmethodentry.h>
-#include "fcitxcore_export.h"
 
 namespace fcitx {
 
@@ -172,18 +172,24 @@ public:
      * @since 5.0.11
      */
     void invokeAction(const InputMethodEntry &entry, InvokeActionEvent &event);
+
+    /**
+     * Process virtual keyboard event
+     *
+     * @param entry input method entry
+     * @param event virtual keyboard event
+     * @since 5.1.0
+     */
+    void virtualKeyboardEvent(const InputMethodEntry &entry,
+                              VirtualKeyboardEvent &VirtualKeyboardEvent);
 };
 
 class FCITXCORE_EXPORT InputMethodEngineV2 : public InputMethodEngine {
 public:
     virtual std::string subModeIconImpl(const InputMethodEntry &,
-                                        InputContext &) {
-        return {};
-    }
+                                        InputContext &);
     virtual std::string subModeLabelImpl(const InputMethodEntry &,
-                                         InputContext &) {
-        return {};
-    }
+                                         InputContext &);
 };
 
 class FCITXCORE_EXPORT InputMethodEngineV3 : public InputMethodEngineV2 {
@@ -192,6 +198,13 @@ public:
                                   InvokeActionEvent &event);
 };
 
+class FCITXCORE_EXPORT InputMethodEngineV4 : public InputMethodEngineV3 {
+public:
+    virtual void
+    virtualKeyboardEventImpl(const InputMethodEntry &entry,
+                             VirtualKeyboardEvent &VirtualKeyboardEvent) = 0;
+};
+
 } // namespace fcitx
 
 #endif // _FCITX_INPUTMETHODENGINE_H_
