@@ -13,8 +13,8 @@
 #include <vector>
 #include <fcitx-utils/connectableobject.h>
 #include <fcitx-utils/macros.h>
+#include <fcitx/fcitxcore_export.h>
 #include <fcitx/inputmethodgroup.h>
-#include "fcitxcore_export.h"
 
 /// \addtogroup FcitxCore
 /// \{
@@ -87,6 +87,15 @@ public:
     /// Simply enumerate input method groups.
     void enumerateGroup(bool forward);
 
+    /**
+     * enumerate group to a certain group.
+     *
+     * For example, current order is A,B,C,D, enumerateGroupTo(C)
+     * will make order to be C,D,A,B.
+     * @arg groupName group name
+     */
+    void enumerateGroupTo(const std::string &groupName);
+
     /**
      * Set default input method for current group.
      *
