@@ -271,4 +271,4 @@ public:
 
 } // namespace fcitx
 
-FCITX_ADDON_FACTORY(fcitx::IMSelectorFactory);
+FCITX_ADDON_FACTORY_V2(imselector, fcitx::IMSelectorFactory);
