@@ -5,7 +5,6 @@
  *
  */
 #include "dbusmenu.h"
-#include "fcitx-utils/log.h"
 #include "fcitx/action.h"
 #include "fcitx/inputcontext.h"
 #include "fcitx/inputmethodentry.h"
@@ -195,7 +194,8 @@ void DBusMenu::fillLayoutItem(
             appendSubItem(subLayoutItems, BII_Restart, depth, propertyNames);
         }
         if (parent_->instance()->canRestart() &&
-            getDesktopType() != DesktopType::DEEPIN) {
+            getDesktopType() != DesktopType::DEEPIN &&
+            getDesktopType() != DesktopType::UKUI) {
             appendSubItem(subLayoutItems, BII_Exit, depth, propertyNames);
         }
     } else if (id == BII_InputMethodGroup) {
