@@ -11,7 +11,8 @@
 #include "xcbmodule.h"
 
 namespace fcitx {
-XCBEventReader::XCBEventReader(XCBConnection *conn) : conn_(conn) {
+XCBEventReader::XCBEventReader(XCBConnection *conn)
+    : conn_(conn), dispatcherToMain_(conn->instance()->eventDispatcher()) {
     postEvent_ =
         conn->instance()->eventLoop().addPostEvent([this](EventSource *source) {
             if (xcb_connection_has_error(conn_->connection())) {
@@ -20,9 +21,9 @@ XCBEventReader::XCBEventReader(XCBConnection *conn) : conn_(conn) {
             }
             FCITX_XCB_DEBUG() << "xcb_flush";
             xcb_flush(conn_->connection());
+            wakeUp();
             return true;
         });
-    dispatcherToMain_.attach(&conn->instance()->eventLoop());
     thread_ = std::make_unique<std::thread>(&XCBEventReader::runThread, this);
 }
 
@@ -50,7 +51,7 @@ bool XCBEventReader::onIOEvent(IOEventFlags flags) {
         hadError_ = true;
         FCITX_WARN() << "XCB connection \"" << conn_->name()
                      << "\" got error: " << err;
-        dispatcherToMain_.schedule([this]() {
+        dispatcherToMain_.scheduleWithContext(watch(), [this]() {
             deferEvent_ =
                 conn_->parent()->instance()->eventLoop().addDeferEvent(
                     [this](EventSource *) {
@@ -72,7 +73,8 @@ bool XCBEventReader::onIOEvent(IOEventFlags flags) {
         hasEvent = !events_.empty();
     }
     if (hasEvent) {
-        dispatcherToMain_.schedule([this]() { conn_->processEvent(); });
+        dispatcherToMain_.scheduleWithContext(
+            watch(), [this]() { conn_->processEvent(); });
     }
     return true;
 }
