@@ -6,16 +6,33 @@
  */
 #include "xcbtraywindow.h"
 #include <unistd.h>
+#include <algorithm>
+#include <cstddef>
+#include <cstdint>
+#include <cstdio>
+#include <cstring>
+#include <string>
+#include <vector>
+#include <cairo.h>
+#include <xcb/xcb.h>
 #include <xcb/xcb_aux.h>
+#include <xcb/xcb_ewmh.h>
 #include <xcb/xcb_icccm.h>
+#include <xcb/xproto.h>
 #include "fcitx-utils/i18n.h"
+#include "fcitx-utils/misc.h"
+#include "fcitx-utils/rect.h"
+#include "fcitx/action.h"
 #include "fcitx/inputcontext.h"
 #include "fcitx/inputmethodentry.h"
 #include "fcitx/inputmethodmanager.h"
 #include "fcitx/statusarea.h"
 #include "fcitx/userinterfacemanager.h"
 #include "common.h"
+#include "xcb_public.h"
 #include "xcbmenu.h"
+#include "xcbui.h"
+#include "xcbwindow.h"
 
 namespace fcitx::classicui {
 
@@ -146,8 +163,9 @@ bool XCBTrayWindow::filterEvent(xcb_generic_event_t *event) {
             createTrayWindow();
             findDock();
             return true;
-        } else if (property->atom == atoms_[ATOM_ORIENTATION] &&
-                   property->window == dockWindow_) {
+        }
+        if (property->atom == atoms_[ATOM_ORIENTATION] &&
+            property->window == dockWindow_) {
             isHorizontal_ = trayOrientation();
             resizeTrayWindow();
             return true;
@@ -353,7 +371,8 @@ void XCBTrayWindow::paint(cairo_t *c) {
 
     cairo_save(c);
     cairo_set_operator(c, CAIRO_OPERATOR_SOURCE);
-    double scaleW = 1.0, scaleH = 1.0;
+    double scaleW = 1.0;
+    double scaleH = 1.0;
     if (image.width() != width() || image.height() != height()) {
         scaleW = static_cast<double>(width()) / image.width();
         scaleH = static_cast<double>(height()) / image.height();
@@ -367,7 +386,8 @@ void XCBTrayWindow::paint(cairo_t *c) {
     int ah = scaleH * image.height();
 
     cairo_scale(c, scaleW, scaleH);
-    cairo_set_source_surface(c, image, (width() - aw) / 2, (height() - ah) / 2);
+    cairo_set_source_surface(c, image, (width() - aw) / 2.0,
+                             (height() - ah) / 2.0);
     cairo_paint(c);
     cairo_restore(c);
 }
@@ -467,19 +487,19 @@ void XCBTrayWindow::updateMenu() {
 
 void XCBTrayWindow::updateGroupMenu() {
     auto &imManager = ui_->parent()->instance()->inputMethodManager();
-    const auto &list = imManager.groups();
+    std::vector<std::string> list = imManager.groups();
     groupActions_.clear();
     for (size_t i = 0; i < list.size(); i++) {
-        auto groupName = list[i];
+        const std::string &groupName = list[i];
         groupActions_.emplace_back();
         auto &groupAction = groupActions_.back();
-        groupAction.setShortText(list[i]);
+        groupAction.setShortText(groupName);
         groupAction.connect<SimpleAction::Activated>(
             [&imManager, groupName](InputContext *) {
                 imManager.setCurrentGroup(groupName);
             });
         groupAction.setCheckable(true);
-        groupAction.setChecked(list[i] == imManager.currentGroup().name());
+        groupAction.setChecked(groupName == imManager.currentGroup().name());
 
         auto &uiManager = ui_->parent()->instance()->userInterfaceManager();
         uiManager.registerAction(&groupAction);
@@ -498,17 +518,17 @@ void XCBTrayWindow::updateInputMethodMenu() {
             return;
         }
         inputMethodActions_.emplace_back();
-        auto imName = entry->uniqueName();
         auto &inputMethodAction = inputMethodActions_.back();
         inputMethodAction.setShortText(entry->name());
         inputMethodAction.connect<SimpleAction::Activated>(
-            [this, imName](InputContext *ic) {
+            [this, imName = entry->uniqueName()](InputContext *ic) {
                 ui_->parent()->instance()->setCurrentInputMethod(ic, imName,
                                                                  false);
             });
         inputMethodAction.setCheckable(true);
         inputMethodAction.setChecked(
-            ic ? (ui_->parent()->instance()->inputMethod(ic) == imName)
+            ic ? (ui_->parent()->instance()->inputMethod(ic) ==
+                  entry->uniqueName())
                : false);
 
         auto &uiManager = ui_->parent()->instance()->userInterfaceManager();
