@@ -4,6 +4,7 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
  */
+#include "fcitx-utils/log.h"
 #include "config.h"
 #include "im/keyboard/isocodes.h"
 
@@ -11,7 +12,7 @@ using namespace fcitx;
 
 int main() {
     IsoCodes isocodes;
-    isocodes.read(ISOCODES_ISO639_JSON, ISOCODES_ISO3166_JSON);
+    isocodes.read(ISOCODES_ISO639_JSON);
     const auto *entry = isocodes.entry("eng");
     FCITX_ASSERT(entry);
     FCITX_ASSERT(entry->iso_639_1_code == "en");
